#include <Arduino.h>
#include <Wire.h>
#include "AHT10.h"

#include <ESP8266WiFi.h>
#include "BlynkSimpleEsp8266.h"


extern "C" {
#include "user_interface.h"
#include "gpio.h"
}


// #define  TIMEOUT (15UL * 60UL * 1000UL)         // Tempertaure triger timeout 
char auth[] = "BLINK_KEY";
// Your WiFi credentials.
// Set password to "" for open networks.
char ssid[] = "WIFI_SSID";
char pass[] = "PASS";

BlynkTimer timer;

void myTimer_Event();
void myTimer_AHT10();


void setup()
{
    Serial.begin(74880);
    Serial.println();

    Blynk.begin(auth, ssid, pass, "blynk.greenhouse.cv.ua", 8080);
    if(!Blynk.isTokenInvalid())
    {
        Serial.println("Blynk server connected!");
        Blynk.syncAll();
        Blynk.syncVirtual(V1);
        Blynk.notify(F("{DEVICE_NAME} power on"));
    }else Serial.println("Blynk server connect error!");
    
    timer.setInterval(1000, myTimer_Event);
    timer.setInterval(2000, myTimer_AHT10);


    Wire.begin();
    if (!AHT10::begin())
    {
        Serial.println(F("AHT10 not detected!"));
        Serial.flush();
        ESP.deepSleep(0);
    }
}

void loop()
{
    Blynk.run();
    timer.run();
}


BLYNK_WRITE(V1)
{
    int pinValue = param.asInt(); // assigning incoming value from pin V1 to a variable
    // You can also use:
    // String i = param.asStr();
    // double d = param.asDouble();
    Serial.print("V1 Slider value is: ");
    Serial.println(pinValue);
}


void myTimer_Event()
{
    Blynk.virtualWrite(V0, millis() / 1000);
    // Serial.println(wifi_get_sleep_type());
    if(analogRead(A0) > 20)
    {
        Blynk.notify("Overvoltage" + String(millis()));
        Serial.println(F("Send notify!"));
    }
}


void myTimer_AHT10()
{
    float temp, hum;
    static uint8_t low_triger = 0;
    static uint8_t higth_triger = 0;
    

    if (AHT10::measure(&temp, &hum))
    {
        Serial.print(F("Temperature is "));
        Serial.print(temp);
        Blynk.virtualWrite(V2, temp);
        Serial.print(F(" C, humidity is "));
        Serial.print(hum);
        Blynk.virtualWrite(V3, hum);
        Serial.println('%');


        if( (temp < 0) && (low_triger == 0) )
        {
            low_triger = 1;
            Blynk.notify("Low temperature" + String(temp));
        }else if( (temp > 1) && (low_triger == 1) ) {
            low_triger = 0;
        }

        if( (temp > 35) && (higth_triger == 0) )
        {
            higth_triger = 1;
            Blynk.notify("Higth temperature" + String(temp));
        }else if( (temp < 34) && (higth_triger == 1) ) 
        {
            higth_triger = 0;
        }
    }
    else
    {
        Serial.println(F("AHT10 read error!"));
    }
}
